#####################################################################
##### Semiparametric Regression for Discrete Time-to-Event Data #####
#####                Electronic Supplement                      #####
#####################################################################
#####     Authors: Moritz Berger and Matthias Schmid            #####
#####################################################################

### Libraries, Functions ###

library("Ecdat")
library("discSurv")
library("mgcv")
source("survivalTree.R")
#--------------------

### Data Preparation ### 

data(UnempDur)
UnempDur$censor1[UnempDur$spell > 20] <- 0
UnempDur$censor2[UnempDur$spell > 20] <- 0
UnempDur$censor3[UnempDur$spell > 20] <- 0
UnempDur$spell[UnempDur$spell > 21]   <- 21
UnempDur[apply(UnempDur[, 2:5], 1, sum) == 0, ] <- NA
UnempDur <- UnempDur[complete.cases(UnempDur), ]
#--------------------

### 1 Introduction ###

# Figure 1  
par(mar=c(5,5,2,1))
barplot(table(UnempDur$spell) / nrow(UnempDur),
        xlab = "observation time to re-employment",
        ylab = "relative frequency", main = " ", 
        ylim = c(0, 0.2), cex.lab = 1.8, cex.axis = 1.5,
        cex.names = 1.5)
#-------------------- 

### Re-Coding of the Data for Model Fitting ### 

UnempDur$spell[UnempDur$spell == 21]  <- 20
UnempDur$status <- 0
UnempDur$status[UnempDur$censor1 == 1 | UnempDur$censor2 == 1 |
                UnempDur$censor3 == 1] <- 1
UnempDur <- UnempDur[, c(1,6:12)]
#-------------------- 

### 5.1 Preprocessing of the Data ### 

head(UnempDur)
#--------------------
table(UnempDur$status)/nrow(UnempDur)
#-------------------- 
UnempDurLong <- dataLong(UnempDur, timeColumn = "spell", 
                         censColumn = "status")
#--------------------
names(UnempDurLong)
#--------------------
UnempDurLong[UnempDurLong$obj == 1, ]
#--------------------
UnempDurLong[UnempDurLong$obj == 12, ]
#--------------------

### 5.2 Regression Modelling ### 

model1 <- glm(formula = y ~ timeInt - 1 +
              age + reprate + disrate + logwage + tenure + ui, 
              data = UnempDurLong, family = binomial(link = "logit"))
#-------------------- 
UnempDurLong$timeIntNum <- as.numeric(UnempDurLong$timeInt)
model2 <- gam(formula = y ~ s(timeIntNum, bs = "ps", k = 5, m = 2) + 
              age + reprate + disrate + logwage + tenure + ui, 
              data = UnempDurLong, family = binomial(link = "logit"))
#--------------------
# Figure 2 
int1 <- coef(model1)[1:20]
bh1  <- exp(int1) / (1 + exp(int1))
obs  <- sapply(1:20, function(i) which(UnempDurLong$timeInt == i)[1])
int2 <- predict(model2, type = "terms")[obs, 7] + coef(model2)[1]
bh2  <- exp(int2) / (1 + exp(int2))
par(mar=c(4,5,1,1))
plot(x = 1:20, y = bh1, pch = 19, 
     xlab = "two-week intervals",
     ylab = expression(paste("exp(", hat(gamma)[ot], ")/
                       (1+exp(", hat(gamma)[ot], ")")),
     xlim = c(0, 20), ylim = c(0, 0.15), 
     cex = 2.5, cex.lab = 1.8, cex.axis = 1.8)
lines(x = 1:20, y = bh2, type = "b", lwd = 5, col = "grey")
points(x = 1:20, y = bh2, pch = 15, cex = 2.5, col = "grey")
#--------------------
model2$sp 
#--------------------
# Figure 3a (left)
lambda_hat   <- predict(model1, type = "response")
quantile_hat <- quantile(lambda_hat, probs=seq(0, 1, by = 0.05))
subset_ID    <- cut(lambda_hat, breaks = quantile_hat)
subsets      <- split(UnempDurLong$y, f = subset_ID)
obs_hazards  <- sapply(subsets, mean)
subsets_l    <- split(lambda_hat, f = subset_ID)
fit_hazards  <- sapply(subsets_l, mean)
par(mar=c(5,6,1,1))
plot(x = fit_hazards, y = obs_hazards, pch = 19, 
     xlab = "fitted hazards", ylab = "relative frequency of events",
     xlim = c(0, 0.3), ylim = c(0, 0.3), cex = 2, cex.lab = 2,
     cex.axis = 2)
lines(x = c(0, 0.3), y = c(0, 0.3), lwd = 3, lty = "dashed")
#--------------------
# Figure 3b (left)
lambda_hat   <- predict(model2, type="response")
quantile_hat <- quantile(lambda_hat, probs=seq(0, 1, by=0.05))
subset_ID    <- cut(lambda_hat, breaks=quantile_hat)
subsets      <- split(UnempDurLong$y, subset_ID)
obs_hazards  <- sapply(subsets, mean)
subsets_l    <- split(lambda_hat, subset_ID)
fit_hazards  <- sapply(subsets_l, mean)
par(mar=c(5,6,1,1))
plot(x = fit_hazards, y = obs_hazards, pch = 19, 
     xlab = "fitted hazards", ylab = "relative frequency of events",
     xlim = c(0, 0.3), ylim = c(0, 0.3), cex = 2, cex.lab = 2,
     cex.axis = 2)
lines(x = c(0, 0.3), y = c(0, 0.3), lwd = 3, lty = "dashed")
#--------------------
# Figure 3a (right)
res_hat   <- martingaleResid(UnempDur, spell ~ ui + reprate +
                             logwage + disrate + tenure, 
                             censColumn = "status",
                             linkFunc = "logit", idColumn = NULL)
res_hat   <- res_hat$Output$MartingaleResid
loessPred <- predict(loess(res_hat ~ UnempDur$age))
par(mar=c(5,6,1,1))
plot(x = UnempDur$age, y = res_hat, pch = 19, col = "grey", 
     xlab = "age", ylab = expression(r), ylim = c(-2.235, 0.95),
     cex = 1.2, cex.lab = 2, cex.axis = 2)
lines(x = UnempDur$age[order(UnempDur$age)],
      y = loessPred[order(UnempDur$age)], lwd = 7)
abline(h = 0, lty = "dashed", lwd = 2)
#--------------------
# Figure 3b (right)
mod_no_age   <- gam(formula = y ~ s(timeIntNum, bs = "ps", k = 5, m = 2) + 
                    ui + reprate + logwage + disrate + tenure, 
                    data = UnempDurLong, family = binomial)
hazards      <- predict(mod_no_age, type = "response")
splitHazards <- split(hazards, UnempDurLong$obj)
splitY       <- split(UnempDurLong$y, UnempDurLong$obj)
res_hat      <- sapply(1:length(splitY), function(x) sum(splitY[[x]]-splitHazards[[x]]))
loessPred    <- predict(loess(res_hat~UnempDur$age))
par(mar=c(5,6,1,1))
plot(x = UnempDur$age, y = res_hat, pch = 19, col = "grey", 
     xlab = "age", ylab = expression(r), ylim = c(-2.24, 0.95),
     cex = 1.2, cex.lab = 2, cex.axis = 2)
lines(x = UnempDur$age[order(UnempDur$age)],
      y = loessPred[order(UnempDur$age)], lwd = 7)
abline(h = 0, lty = "dashed", lwd = 2)
#--------------------
model3 <- gam(formula = y ~ s(timeIntNum, bs = "ps", k = 5) +
              s(age, bs = "ps", k = 25) +
              reprate + disrate + logwage + tenure + ui, 
              data = UnempDurLong, family = binomial(link = "logit"))
#--------------------
model3$sp["s(age)"]
#--------------------
# Figure 4 
par(mar=c(4.5,5,1,1))
plot(x = model3, select = 2, rug = FALSE, shade = TRUE, shade.col = "gray85", 
     xlab = "age", ylab = "f(age)", xlim = c(20,60), ylim=c(-0.6,0.6),
     lwd = 5, cex.lab = 1.8, cex.axis = 1.8)
abline(h = 0, lty = "dashed", lwd = 2)
#--------------------

### 5.3 Tree-Based Modeling ### 

# Libraries
library("rpart")
library("sampling")
library("partykit")
#--------------------

# Note: calculation of model4 takes some time 
model4 <- survivalTree(formula = y ~ timeInt + age +
                       reprate + disrate + logwage + tenure + ui,
                       data = UnempDur, tuning = "BIC", 
                       timeColumn = "spell", censColumn = "status",
                       minimal_ns = seq(100, 1500, by = 10),
                       trace = TRUE)
#--------------------
# Figure 5 
par(mar=c(4,5,1,1))
plot(x = seq(100, 1500, by = 10), y = model4$crit, type = "l",
     lwd = 4, xlab = "minimal node size", ylab = "BIC", 
     cex.lab = 1.5, cex.axis = 1.5)
abline(v = seq(100, 1500, by = 10)[which.min(model4$crit)],
       lty = "dashed", lwd = 3)
#--------------------
# Figure 6
par(mar=c(1,1,1,1))
plot(model4$model)
text(model4$model, cex = 1.3)
#--------------------

### 5.4 Comparison and Model Choice ### 

# Function to compute the log-likelihood 
loglikelihood <- function(model, data, type){
  lambda_hat <- predict(model, newdata = data, type = type)
  l          <- 0
  for(i in 1:nrow(data)){
    l <- l + data$y[i]*log(lambda_hat[i])+(1-data$y[i])*log(1-lambda_hat[i])
  }
  return(l)
}
#--------------------
# Function for stratified sampling by spell 
require(sampling)
stratified <- function(df, group, size) {
  temp <- df[order(df[group]), ]
  if (size < 1) {
    size <- ceiling(table(temp[group]) * size)
  } else if (size >= 1) {
    size <- rep(size, times = length(table(temp[group])))
  }
  strat <- strata(temp, stratanames = names(temp[group]),
                  size = size, method = "srswor")
  (dsample <- getdata(temp, strat))
}
#--------------------
# Function to perform one simulation run 
one_sim <- function(seed, df){
  df$ID <- 1:nrow(df)
  
  # sampling 
  set.seed(seed)
  trainShort  <- stratified(df, 1, .8)
  testShort   <- df[!(df$ID %in% trainShort$ID), ]
  trainLong   <- dataLong(trainShort, "spell", "status")
  testLong    <- dataLong(testShort, "spell", "status")
  
  # estimation 
  mod1 <- glm(formula = y ~ timeInt - 1 + 
              age + ui + reprate + logwage + disrate + tenure, 
              data = trainLong, family = binomial)
  trainLong$timeInt <- as.numeric(trainLong$timeInt)
  testLong$timeInt  <- as.numeric(testLong$timeInt)
  mod2 <- gam(formula = y ~ s(timeInt, bs = "ps", k = 5, m = 2) + 
              age + ui + reprate + logwage + disrate + tenure, 
              data = trainLong, family = binomial)
  mod3 <- gam(formula = y ~ s(timeInt, bs = "ps", k = 5, m = 2) + 
              s(age, bs = "ps", k = 25, m = 2) + 
              ui + reprate + logwage + disrate + tenure, 
              data = trainLong, family = binomial)
  mod4 <- survivalTree(formula = y ~ timeInt + age + 
                       ui + reprate + logwage + disrate + tenure, 
                       data = trainShort, tuning = "BIC", 
                       timeColumn = "spell", censColumn = "status",
                       minimal_ns = seq(200, 1200, by = 10), 
                       trace = TRUE)
  
  # evaluation 
  ll    <- numeric(4)
  ll[1] <- loglikelihood(model = mod1, data = testLong, type = "response")
  ll[2] <- loglikelihood(model = mod2, data = testLong, type = "response")
  ll[3] <- loglikelihood(model = mod3, data = testLong, type = "response")
  ll[4] <- loglikelihood(model = mod4$model, data = testLong, type = "vector")
  
  return(ll)
}
#--------------------
# Run the whole simulation (Note: takes several hours!)
# The result is stored in the file 'prediction.rda'.
res <- matrix(NA, nrow = 100, ncol = 4)
for(i in 1:100){
  res[i,] <- one_sim(seed = i, df = UnempDur)
}
# save(res, file="prediction.rda")
#--------------------
# Figure 7
load("prediction.rda")
res <- -2*res
par(mar=c(3,5,1,1))
boxplot(x = res, names = c("model1", "model2", "model3", "model4"), 
        col = grey(0.8), pch = 21, bg = grey(0.8),
        ylab = "predictive deviance", cex.lab = 1.8, cex.axis = 1.8)
abline(h = median(res[,4]), lty = "dashed", lwd = 3)
#--------------------

