survivalTree <- function(formula, data, tuning=c("AIC", "BIC", "ll"),
                         timeColumn, censColumn, minimal_ns,
                         trace = FALSE){
  
  # formula: model formula, like for lm() or glm(), must be of the form 'y ~ timeInt + ...' 
  # data: data in short format 
  # tuning: AIC, BIC or predictive log-likelihood
  # timeColumn, censColumn: for dataLong() 
  # minimal_ns: sequence of minimal node sizes (tuning parameters)
  
  require(rpart)
  require(discSurv)

  dataShortAll <- data
  dataLongAll  <- dataLong(data, timeColumn, censColumn,
                           timeAsFactor = FALSE)
  
  form <- as.formula(formula)
  
  if(missing(minimal_ns)){
    minimal_ns <- 1:(floor(nrow(dataLongAll) / 2))
  }
  
  # Laplace correction
  laplace <- function(data, model, where){
    p <- rep(NA, length(where))
    for (ii in 1:length(where)){
      ind <- where[ii]
      Ncount <- length(data$y[model$where == ind]) + 2
      count1 <- sum(data$y[model$where == ind]) + 1
      p[ii] <- count1 / Ncount
    }
    return(p)
  }
  
  # One Tree
  one_tree <- function(mb, data){
    
    model <- rpart(form, data = data,
                   control = rpart.control(cp = 0, minbucket = mb),
                   model = TRUE)
    
    p <- laplace(data, model, model$where)

    ll  <- sum(log(p) * data$y + log(1 - p) * (1 - data$y))
    nsplits <- max(model$cptable[,2])
    AIC <- - 2 * ll + 2 * nsplits
    BIC <- - 2 * ll + log(nrow(data)) * nsplits
    
    return(list("model" = model, "AIC" = AIC, "BIC" = BIC, "p" = p))
  }
  
  # code for stratified sampling
  require(sampling)
  stratified <- function(df, group, size) {
    
    temp <- df[order(df[group]),]
    if (size < 1) {
      size <- ceiling(table(temp[group]) * size)
    } else if (size >= 1) {
      size <- rep(size, times = length(table(temp[group])))
    }
    
    strat <- strata(temp, stratanames = names(temp[group]),
                    size = size, method = "srswor")
    (dsample <- getdata(temp, strat))
  }
  
  crit <- numeric(length(minimal_ns))
  
  # AIC
  if(tuning == "AIC"){
    k <- 0
    for (MB in minimal_ns){
      k   <- k + 1
      est <- one_tree(MB, dataLongAll)
      crit[k] <- est$AIC
      if(trace & k == 1){
        cat("minimal node size:\n")
      }
      if(trace){
        cat(MB, "\n")
      }
    }
  }
  # BIC 
  if(tuning == "BIC"){
    k <- 0
    for (MB in minimal_ns){
      k <- k + 1
      est <- one_tree(MB, dataLongAll)
      crit[k] <- est$BIC
      if(trace & k == 1){
        cat("minimal node size:\n")
      }
      if(trace){
        cat(MB, "\n")
      }
    }
  }
  # predictive loglikelihood
  if(tuning == "ll"){
    require(partykit)
    for(cv in 1:5){
      if(trace){print(cv)}
      dataShortAll$ID <- 1:nrow(dataShortAll)
      set.seed(cv)
      trainShort5 <- stratified(dataShortAll, 1, .8)
      testShort5  <- dataShortAll[!(dataShortAll$ID %in%
                                    trainShort5$ID),]
      trainLong5  <- dataLong(trainShort5, timeColumn, censColumn,
                              timeAsFactor = FALSE)
      testLong5   <- dataLong(testShort5, timeColumn, censColumn,
                              timeAsFactor = FALSE)
      ll2 <- numeric(length(crit))
      k <- 0 
      for (MB in minimal_ns){
        k <- k + 1
        est_train  <- one_tree(MB, trainLong5)
        p_train    <- est_train$p
        nsplits    <- max(est_train$model$cptable[,2])
        where_test <- ifelse(nsplits>0, predict(as.party(est_train$model),
                                                newdata = testLong5, type = "node"), rep(1,nrow(testLong5)))
        p_test     <- numeric(length(where_test))
        for(l in unique(where_test)){
          p_test[where_test == l] <-
            unique(p_train[est_train$model$where == l])
        }
        ll2[k]     <- sum(log(p_test) * testLong5$y +
                          log(1 - p_test) * (1 - testLong5$y))
        if(trace & k == 1){
          cat("minimal node size:\n")
        }
        if(trace){
          cat(MB, "\n")
        }
      }
      crit <- crit + ll2 
    }
    crit <- -crit 
  }
  
  final_mb    <- minimal_ns[which.min(crit)]
  final_model <- one_tree(final_mb, dataLongAll)$model
  
  leaves      <- sort(unique(final_model$where))
  final_p     <- laplace(dataLongAll, final_model, leaves)
  # incorporate Laplace correction in rpart 
  final_model$frame$yval[leaves] <- final_p

  return(list("model" = final_model,
              "crit" = crit))
}